@@ -14,6 +14,7 @@
 #include "oj.h"
 #include "parse.h"
 #include "resolve.h"
+#include "trace.h"
 
 extern void	oj_set_obj_ivar(Val parent, Val kval, VALUE value);
 extern VALUE	oj_parse_xml_time(const char *str, int len); // from object.c
@@ -463,6 +464,9 @@ dump_common(VALUE obj, int depth, Out out) {
 	const char	*s;
 	int		len;
 
+	if (Yes == out->opts->trace) {
+	    oj_trace("to_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyIn);
+	}
 #if HAS_METHOD_ARITY
 	if (0 == rb_obj_method_arity(obj, oj_to_json_id)) {
 	    rs = rb_funcall(obj, oj_to_json_id, 0);
@@ -472,6 +476,9 @@ dump_common(VALUE obj, int depth, Out out) {
 #else
 	rs = rb_funcall2(obj, oj_to_json_id, out->argc, out->argv);
 #endif
+	if (Yes == out->opts->trace) {
+	    oj_trace("to_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyOut);
+	}
 	s = rb_string_value_ptr((VALUE*)&rs);
 	len = (int)RSTRING_LEN(rs);
 
@@ -482,6 +489,9 @@ dump_common(VALUE obj, int depth, Out out) {
     } else if (Yes == out->opts->as_json && rb_respond_to(obj, oj_as_json_id)) {
 	volatile VALUE	aj;
 
+	if (Yes == out->opts->trace) {
+	    oj_trace("as_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyIn);
+	}
 	// Some classes elect to not take an options argument so check the arity
 	// of as_json.
 #if HAS_METHOD_ARITY
@@ -493,6 +503,9 @@ dump_common(VALUE obj, int depth, Out out) {
 #else
 	aj = rb_funcall2(obj, oj_as_json_id, out->argc, out->argv);
 #endif
+	if (Yes == out->opts->trace) {
+	    oj_trace("as_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyOut);
+	}
 	// Catch the obvious brain damaged recursive dumping.
 	if (aj == obj) {
 	    volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
@@ -874,6 +887,9 @@ void
 oj_dump_custom_val(VALUE obj, int depth, Out out, bool as_ok) {
     int	type = rb_type(obj);
 
+    if (Yes == out->opts->trace) {
+	oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceIn);
+    }
     if (MAX_DEPTH < depth) {
 	rb_raise(rb_eNoMemError, "Too deeply nested.\n");
     }
@@ -882,10 +898,16 @@ oj_dump_custom_val(VALUE obj, int depth, Out out, bool as_ok) {
 
 	if (NULL != f) {
 	    f(obj, depth, out, true);
+	    if (Yes == out->opts->trace) {
+		oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceOut);
+	    }
 	    return;
 	}
     }
     oj_dump_nil(Qnil, depth, out, false);
+    if (Yes == out->opts->trace) {
+	oj_trace("dump", Qnil, __FILE__, __LINE__, depth, TraceOut);
+    }
 }
 
 ///// load functions /////
@@ -946,6 +968,9 @@ hash_set_cstr(ParseInfo pi, Val kval, const char *str, size_t len, const char *o
 	default:
 	    break;
 	}
+	if (Yes == pi->options.trace) {
+	    oj_trace_parse_call("set_string", pi, __FILE__, __LINE__, rstr);
+	}
     }
 }
 
@@ -963,6 +988,9 @@ end_hash(struct _ParseInfo *pi) {
 	}
 	parent->clas = Qundef;
     }
+    if (Yes == pi->options.trace) {
+	oj_trace_parse_hash_end(pi, __FILE__, __LINE__);
+    }
 }
 
 static VALUE
@@ -981,11 +1009,12 @@ calc_hash_key(ParseInfo pi, Val parent) {
 
 static void
 hash_set_num(struct _ParseInfo *pi, Val kval, NumInfo ni) {
-    Val	parent = stack_peek(&pi->stack);
+    Val			parent = stack_peek(&pi->stack);
+    volatile VALUE	rval = oj_num_as_value(ni);
 
     switch (rb_type(parent->val)) {
     case T_OBJECT:
-	oj_set_obj_ivar(parent, kval, oj_num_as_value(ni));
+	oj_set_obj_ivar(parent, kval, rval);
 	break;
     case T_HASH:
 	if (4 == parent->klen && NULL != parent->key && rb_cTime == parent->clas && 0 == strncmp("time", parent->key, 4)) {
@@ -1001,7 +1030,7 @@ hash_set_num(struct _ParseInfo *pi, Val kval, NumInfo ni) {
 	    if (86400 == ni->exp) { // UTC time
 		parent->val = rb_time_nano_new(ni->i, (long)nsec);
 		// Since the ruby C routines alway create local time, the
-		// offset and then a convertion to UTC keeps makes the time
+		// offset and then a conversion to UTC keeps makes the time
 		// match the expected value.
 		parent->val = rb_funcall2(parent->val, oj_utc_id, 0, 0);
 	    } else if (ni->hasExp) {
@@ -1020,13 +1049,17 @@ hash_set_num(struct _ParseInfo *pi, Val kval, NumInfo ni) {
 	    } else {
 		parent->val = rb_time_nano_new(ni->i, (long)nsec);
 	    }
+	    rval = parent->val;
 	} else {
-	    rb_hash_aset(parent->val, calc_hash_key(pi, kval), oj_num_as_value(ni));
+	    rb_hash_aset(parent->val, calc_hash_key(pi, kval), rval);
 	}
 	break;
     default:
 	break;
     }
+    if (Yes == pi->options.trace) {
+	oj_trace_parse_call("set_string", pi, __FILE__, __LINE__, rval);
+    }
 }
 
 static void
@@ -1043,13 +1076,20 @@ hash_set_value(ParseInfo pi, Val kval, VALUE value) {
     default:
 	break;
     }
+    if (Yes == pi->options.trace) {
+	oj_trace_parse_call("set_value", pi, __FILE__, __LINE__, value);
+    }
 }
 
 static void
 array_append_num(ParseInfo pi, NumInfo ni) {
-    Val	parent = stack_peek(&pi->stack);
+    Val			parent = stack_peek(&pi->stack);
+    volatile VALUE	rval = oj_num_as_value(ni);
     
-    rb_ary_push(parent->val, oj_num_as_value(ni));
+    rb_ary_push(parent->val, rval);
+    if (Yes == pi->options.trace) {
+	oj_trace_parse_call("append_number", pi, __FILE__, __LINE__, rval);
+    }
 }
 
 static void
@@ -1066,6 +1106,9 @@ array_append_cstr(ParseInfo pi, const char *str, size_t len, const char *orig) {
 	}
     }
     rb_ary_push(stack_peek(&pi->stack)->val, rstr);
+    if (Yes == pi->options.trace) {
+	oj_trace_parse_call("append_string", pi, __FILE__, __LINE__, rstr);
+    }
 }
 
 void
