@@ -6,6 +6,7 @@
 #include "code.h"
 #include "dump.h"
 #include "rails.h"
+#include "trace.h"
 
 // Workaround in case INFINITY is not defined in math.h or if the OS is CentOS
 #define OJ_INFINITY (1.0/0.0)
@@ -119,6 +120,9 @@ dump_to_json(VALUE obj, Out out) {
     const char		*s;
     int			len;
 
+    if (Yes == out->opts->trace) {
+	oj_trace("to_json", obj, __FILE__, __LINE__, 0, TraceRubyIn);
+    }
 #if HAS_METHOD_ARITY
     if (0 == rb_obj_method_arity(obj, oj_to_json_id)) {
 	rs = rb_funcall(obj, oj_to_json_id, 0);
@@ -128,6 +132,9 @@ dump_to_json(VALUE obj, Out out) {
 #else
     rs = rb_funcall2(obj, oj_to_json_id, out->argc, out->argv);
 #endif
+    if (Yes == out->opts->trace) {
+	oj_trace("to_json", obj, __FILE__, __LINE__, 0, TraceRubyOut);
+    }
 
     s = rb_string_value_ptr((VALUE*)&rs);
     len = (int)RSTRING_LEN(rs);
@@ -464,10 +471,26 @@ time_alt(VALUE obj, int depth, Out out) {
 	{ "n", 1, Qundef, 0, Qundef },
 	{ NULL, 0, Qnil },
     };
-    struct timespec	ts = rb_time_timespec(obj);
+    time_t	sec;
+    long long	nsec;
 
-    attrs[0].num = ts.tv_sec;
-    attrs[1].num = ts.tv_nsec;
+#if HAS_RB_TIME_TIMESPEC
+    {
+	struct timespec	ts = rb_time_timespec(obj);
+	sec = ts.tv_sec;
+	nsec = ts.tv_nsec;
+    }
+#else
+    sec = rb_num2ll(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
+#if HAS_NANO_TIME
+    nsec = rb_num2ll(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
+#else
+    nsec = rb_num2ll(rb_funcall2(obj, oj_tv_usec_id, 0, 0)) * 1000;
+#endif
+#endif
+
+    attrs[0].num = sec;
+    attrs[1].num = nsec;
 
     oj_code_attrs(obj, attrs, depth, out, true);
 }
@@ -901,7 +924,10 @@ set_state_depth(VALUE state, int depth) {
 void
 oj_dump_compat_val(VALUE obj, int depth, Out out, bool as_ok) {
     int	type = rb_type(obj);
-    
+
+    if (Yes == out->opts->trace) {
+	oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceIn);
+    }
     if (out->opts->dump_opts.max_depth <= depth) {
 	// When JSON.dump is called then an ArgumentError is expected and the
 	// limit is the depth inclusive. If JSON.generate is called then a
@@ -924,8 +950,14 @@ oj_dump_compat_val(VALUE obj, int depth, Out out, bool as_ok) {
 
 	if (NULL != f) {
 	    f(obj, depth, out, as_ok);
+	    if (Yes == out->opts->trace) {
+		oj_trace("dump", obj, __FILE__, __LINE__, depth, TraceOut);
+	    }
 	    return;
 	}
     }
     oj_dump_nil(Qnil, depth, out, false);
+    if (Yes == out->opts->trace) {
+	oj_trace("dump", Qnil, __FILE__, __LINE__, depth, TraceOut);
+    }
 }
