@@ -44,13 +44,13 @@ static VALUE	state_class;
 /* Document-method: parser=
  * call-seq: parser=(parser)
  * 
- * Does nothing other than provide compatibiltiy.
+ * Does nothing other than provide compatibility.
  * - *parser* [_Object_] ignored
  */
 /* Document-method: generator=
  * call-seq: generator=(generator)
  * 
- * Does nothing other than provide compatibiltiy.
+ * Does nothing other than provide compatibility.
  * - *generator* [_Object_] ignored
  */
 
@@ -370,6 +370,7 @@ mimic_generate_core(int argc, VALUE *argv, Options copts) {
     // it is.
     copts->dump_opts.nan_dump = RaiseNan;
     copts->mode = CompatMode;
+    copts->to_json = Yes;
     if (2 == argc && Qnil != argv[1]) {
 	oj_parse_mimic_dump_options(argv[1], copts);
     }
@@ -606,7 +607,7 @@ mimic_parse_bang(int argc, VALUE *argv, VALUE self) {
 /* Document-method: recurse_proc
  * call-seq: recurse_proc(obj, &proc)
  * 
- * Yields to the proc for every element in the obj recursivly.
+ * Yields to the proc for every element in the obj recursively.
  * 
  * - *obj* [_Hash_|Array] object to walk
  * - *proc* [_Proc_] to yield to on each element
@@ -684,6 +685,7 @@ static struct _Options	mimic_object_to_json_options = {
     No,		// allow_invalid
     No,		// create_ok
     No,		// allow_nan
+    No,		// trace
     oj_json_class,// create_id
     10,		// create_id_len
     3,		// sec_prec
@@ -857,9 +859,6 @@ oj_define_mimic_json(int argc, VALUE *argv, VALUE self) {
 	    rb_funcall2(Oj, rb_intern("mimic_loaded"), 0, 0);
 	}
     }
-
-    // TBD create all modules in mimic_loaded
-
     oj_mimic_json_methods(json);
 
     rb_define_method(rb_cObject, "to_json", mimic_object_to_json, -1);
