@@ -145,6 +145,7 @@ class Juice < Minitest::Test
       :time_format=>:unix_zone,
       :bigdecimal_load=>:float,
       :create_id=>'classy',
+      :create_additions=>true,
       :space=>'z',
       :array_nl=>'a',
       :object_nl=>'o',
@@ -155,6 +156,7 @@ class Juice < Minitest::Test
       :allow_nan=>true,
       :array_class=>Array,
       :ignore=>nil,
+      :trace=>true,
     }
     Oj.default_options = alt
     #keys = alt.keys
@@ -405,6 +407,12 @@ class Juice < Minitest::Test
     assert_equal('null', json)
   end
 
+  def test_time_neg
+    t = Time.parse("1900-01-01 00:18:59 UTC")
+    json = Oj.dump(t, :mode => :custom, :time_format => :unix)
+    assert_equal('-2208987661.000000000', json)
+  end
+
   # Class
   def test_class_null
     json = Oj.dump(Juice, :mode => :null)
